package java.security.cert;

import sun.misc.IOUtils;
import sun.security.util.ObjectIdentifier;
import java.io.ObjectInputStream;
import java.util.Iterator;
import java.io.ObjectOutputStream;
import java.io.IOException;
import sun.security.x509.InvalidityDateExtension;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import javax.security.auth.x500.X500Principal;
import java.util.Date;

public class CertificateRevokedException extends CertificateException
{
    private static final long serialVersionUID = 7839996631571608627L;
    private Date revocationDate;
    private final CRLReason reason;
    private final X500Principal authority;
    private transient Map<String, Extension> extensions;
    
    public CertificateRevokedException(Date date, CRLReason reason, X500Principal authority, Map<String, Extension> map) {
        if (date == null || reason == null || authority == null || map == null) {
            throw new NullPointerException();
        }
        this.revocationDate = new Date(date.getTime());
        this.reason = reason;
        this.authority = authority;
        (this.extensions = Collections.checkedMap(new HashMap(), String.class, Extension.class)).putAll(map);
        return;
    }
    
    public Date getRevocationDate() {
        return (Date)this.revocationDate.clone();
    }
    
    public CRLReason getRevocationReason() {
        return this.reason;
    }
    
    public X500Principal getAuthorityName() {
        return this.authority;
    }
    
    public Date getInvalidityDate() {
        Extension ext = (Extension)this.getExtensions().get("2.5.29.24");
        if (ext == null) {
            return null;
        }
        try {
            return new Date(InvalidityDateExtension.toImpl(ext).get("DATE").getTime());
        }
        catch (IOException ex) {
            return null;
        }
    }
    
    public Map<String, Extension> getExtensions() {
        return Collections.unmodifiableMap(this.extensions);
    }
    
    @Override
    public String getMessage() {
        return "Certificate has been revoked, reason: " + this.reason + ", revocation date: " + this.revocationDate + ", authority: " + this.authority + ", extension OIDs: " + this.extensions.keySet();
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeInt(this.extensions.size());
        Iterator<Map.Entry<String, Extension>> iterator = this.extensions.entrySet().iterator();
        while (iterator.hasNext()) {
            Extension extension = (Extension)((Map.Entry)iterator.next()).getValue();
            objectOutputStream.writeObject(extension.getId());
            objectOutputStream.writeBoolean(extension.isCritical());
            byte[] value = extension.getValue();
            objectOutputStream.writeInt(value.length);
            objectOutputStream.write(value);
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.revocationDate = new Date(this.revocationDate.getTime());
        int int1 = objectInputStream.readInt();
        if (int1 == 0) {
            this.extensions = Collections.emptyMap();
        }
        else {
            if (int1 < 0) {
                throw new IOException("size cannot be negative");
            }
            this.extensions = new HashMap((int1 > 20) ? 20 : int1);
        }
        for (int i = 0; i < int1; ++i) {
            String oid = (String)objectInputStream.readObject();
            this.extensions.put(oid, sun.security.x509.Extension.newExtension(new ObjectIdentifier(oid), objectInputStream.readBoolean(), IOUtils.readExactlyNBytes(objectInputStream, objectInputStream.readInt())));
        }
    }
}
